#pragma once
#include "communication/bridge/proxy/i_bridge_proxy.h"
#include "communication/bridge/client/bridge_client.h"
#include "communication/bridge/client/i_xmlrpc_client.h"
#include "communication/bridge/client/xmlrpc_client.h"
#include "communication/bridge/connection_lifecycle/i_connection_state_machine.h"

constexpr int PORT = 8088;

namespace spin
{
    class BridgeProxy : public IBridgeProxy
    {
    public:
        BridgeProxy(std::shared_ptr<IBridgeClient> client, std::shared_ptr<IConnectionStateMachine> stateMachine);
        BridgeProxy(std::shared_ptr<IBridgeClient> client);

        void connectTo(const std::string &host) const override;
        void disconnect() const override;
        void connectFromCommandTo(const std::string &host) const override;
        
        void enableExhibitionMode() const override;
        
        void acknowledgeErrors() const override;
        void stopMotor() const override;
        void startMotor(const int speed_rpm, const int rampTime_ms = 500) const override;
        void stopSafetyShield(const bool blocking = true) const override;
        void doSafetyShieldHoming(const bool blocking = true) const override;
        void lockBit(const bool blocking = true) const override;
        void releaseBit(const bool blocking = true) const override;
        void coverBit(const bool blocking = true) const override;
        void coverScrew(const double &length, const bool blocking = true) const override;
        void moveSafetyShieldTo(const Length &length, const bool blocking = true) const override;
        void followRobot() const override;
        UUID startScrewProgram(const UUID &programUuid) const override;
        std::string programMessages() const override;
        std::set<Program> programs() const override;
        std::set<Group> groups() const override;
        Speed drivingSpeedSetpoint() const override;
        UUID latestResultUUID() const override;
        ProgramResult::Evaluation latestResultEvaluation() const override;

        std::optional<BridgeInfo> bridgeInfo() const override;
        std::optional<ToolInfo> toolInfo() const override;

    private:
        void waitForIdleSafetyShield() const;
        std::shared_ptr<IBridgeClient> client_;
        std::shared_ptr<IConnectionStateMachine> stateMachine_;
    };

} // namespace spin
